/* *****************************************************************************
 *
 * $RCSfile: $
 * $Revision: $
 *
 * This module provides interface routines to the LPC ARM UARTs.
 * Copyright 2004, R O SoftWare
 * No guarantees, warrantees, or promises, implied or otherwise.
 * May be used for hobby or commercial purposes provided copyright
 * notice remains intact.
 *
 * reduced to see what has to be done for minimum UART-support by mthomas
 * added uart1 and some changes by holger klabunde.
 *****************************************************************************/

// #warning "this is a reduced version of the R O Software code"

#include <lpc213x.h>
#include "uart.h"

/* on LPC210x: UART0 TX-Pin=P0.2, RX-Pin=P0.1 
   PINSEL0 has to be set to "UART-Function" = Function "01" 
   for Pin 0.0 and 0.1 */
   
#define PINSEL_TXD0  0
#define PINSEL_RXD0  2
#define PINSEL_UART0_FUNC   1

// Values of Bits in PINSEL to activate UART0
#define UART0_PINSEL    ((PINSEL_UART0_FUNC<<PINSEL_TXD0) | (PINSEL_UART0_FUNC<<PINSEL_RXD0))
#define UART0_PINMASK   ((3<<PINSEL_TXD0) | (3<<PINSEL_RXD0))  /* PINSEL0 Mask for UART0 */

// U0_LCR devisor latch bit 
#define UART0_LCR_DLAB  7

#define PINSEL_TXD1  16
#define PINSEL_RXD1  18
#define PINSEL_UART1_FUNC   1

// Values of Bits in PINSEL to activate UART0
#define UART1_PINSEL    ((PINSEL_UART1_FUNC<<PINSEL_TXD1) | (PINSEL_UART1_FUNC<<PINSEL_RXD1))
#define UART1_PINMASK   ((3<<PINSEL_TXD1) | (3<<PINSEL_RXD1))  /* PINSEL0 Mask for UART1 */

// U1_LCR devisor latch bit 
#define UART1_LCR_DLAB  7

/*    baudrate divisor - use UART_BAUD macro
 *    mode - see typical modes (uart.h)
 *    fmode - see typical fmodes (uart.h)
 *    NOTE: uart0Init(UART_BAUD(9600), UART_8N1, UART_FIFO_8); 
 */
void uart0Init(U16 baud, U08 mode, U08 fmode)
{
  // setup Pin Function Select Register (Pin Connect Block) 
  // make sure old values of Bits 0-4 are masked out and
  // set them according to UART0-Pin-Selection
  PINSEL0 = (PINSEL0 & ~UART0_PINMASK) | UART0_PINSEL;

  UART0_IER = 0x00;             // disable all interrupts
  UART0_IIR = 0x00;             // clear interrupt ID register
  UART0_LSR = 0x00;             // clear line status register

  // set the baudrate - DLAB must be set to access DLL/DLM
  UART0_LCR = (1<<UART0_LCR_DLAB); // set divisor latches (DLAB)
  UART0_DLL = (U08)baud;         // set for baud low byte
  UART0_DLM = (U08)(baud >> 8);  // set for baud high byte
  
  // set the number of characters and other
  // user specified operating parameters
  // Databits, Parity, Stopbits - Settings in Line Control Register
  UART0_LCR = (mode & ~(1<<UART0_LCR_DLAB)); // clear DLAB "on-the-fly"
  // setup FIFO Control Register (fifo-enabled + xx trig) 
  UART0_FCR = fmode;
}

void uart0_sendchar(char c)
{
  uart0Putch(c);
}

int uart0Putch(int ch)
{
  while (!(UART0_LSR & ULSR_THRE))          // wait for TX buffer to empty
    continue;                           // also either WDOG() or swap()

  UART0_THR = (U08)ch;  // put char to Transmit Holding Register
  return (U08)ch;      // return char ("stdio-compatible"?)
}

const char *uart0Puts(const char *string)
{
	char ch;
	
	while ((ch = *string)) {
		if (uart0Putch(ch)<0) break;
		string++;
	}
	
	return string;
}

int uart0TxEmpty(void)
{
  return (UART0_LSR & (ULSR_THRE | ULSR_TEMT)) == (ULSR_THRE | ULSR_TEMT);
}

void uart0TxFlush(void)
{
  UART0_FCR |= UFCR_TX_FIFO_RESET;          // clear the TX fifo
}


/* Returns: character on success, -1 if no character is available */
int uart0Getch(void)
{
  if (UART0_LSR & ULSR_RDR)                 // check if character is available
    return UART0_RBR;                       // return character

  return -1;
}

/*    baudrate divisor - use UART_BAUD macro
 *    mode - see typical modes (uart.h)
 *    fmode - see typical fmodes (uart.h)
 *    NOTE: uart1Init(UART_BAUD(9600), UART_8N1, UART_FIFO_8); 
 */
void uart1Init(U16 baud, U08 mode, U08 fmode)
{
  // setup Pin Function Select Register (Pin Connect Block) 
  // make sure old values of Bits are masked out and
  // set them according to UART1-Pin-Selection
  PINSEL0 = (PINSEL0 & ~UART1_PINMASK) | UART1_PINSEL;

  UART1_IER = 0x00;             // disable all interrupts
  UART1_IIR = 0x00;             // clear interrupt ID register
  UART1_LSR = 0x00;             // clear line status register

  // set the baudrate - DLAB must be set to access DLL/DLM
  UART1_LCR = (1<<UART1_LCR_DLAB); // set divisor latches (DLAB)
  UART1_DLL = (U08)baud;         // set for baud low byte
  UART1_DLM = (U08)(baud >> 8);  // set for baud high byte
  
  // set the number of characters and other
  // user specified operating parameters
  // Databits, Parity, Stopbits - Settings in Line Control Register
  UART1_LCR = (mode & ~(1<<UART1_LCR_DLAB)); // clear DLAB "on-the-fly"
  // setup FIFO Control Register (fifo-enabled + xx trig) 
  UART1_FCR = fmode;
}

void uart1_sendchar(char c)
{
  uart1Putch(c);
}

int uart1Putch(int ch)
{
  while (!(UART1_LSR & ULSR_THRE))          // wait for TX buffer to empty
    continue;                           // also either WDOG() or swap()

  UART1_THR = (U08)ch;  // put char to Transmit Holding Register
  return (U08)ch;      // return char ("stdio-compatible"?)
}

const char *uart1Puts(const char *string)
{
	char ch;
	
	while ((ch = *string)) {
		if (uart1Putch(ch)<0) break;
		string++;
	}
	
	return string;
}

int uart1TxEmpty(void)
{
  return (UART1_LSR & (ULSR_THRE | ULSR_TEMT)) == (ULSR_THRE | ULSR_TEMT);
}

void uart1TxFlush(void)
{
  UART1_FCR |= UFCR_TX_FIFO_RESET;          // clear the TX fifo
}


/* Returns: character on success, -1 if no character is available */
int uart1Getch(void)
{
  if (UART1_LSR & ULSR_RDR)                 // check if character is available
    return UART1_RBR;                       // return character

  return -1;
}

